import { showErrorMessage } from '@jupyterlab/apputils';
class JupyterlabCodeFormatter {
    constructor(client) {
        this.working = false;
        this.client = client;
    }
    formatCode(code, formatter, options, notebook, cache) {
        return this.client
            .request('format' + (cache ? '?cached' : ''), 'POST', JSON.stringify({
            code,
            notebook,
            formatter,
            options
        }))
            .then(resp => JSON.parse(resp));
    }
}
export class JupyterlabNotebookCodeFormatter extends JupyterlabCodeFormatter {
    constructor(client, notebookTracker) {
        super(client);
        this.notebookTracker = notebookTracker;
    }
    async formatAction(config, formatter) {
        return this.formatCells(true, config, formatter);
    }
    async formatSelectedCodeCells(config, formatter, notebook) {
        return this.formatCells(true, config, formatter, notebook);
    }
    async formatAllCodeCells(config, formatter, notebook) {
        return this.formatCells(false, config, formatter, notebook);
    }
    getCodeCells(selectedOnly = true, notebook) {
        if (!this.notebookTracker.currentWidget) {
            return [];
        }
        const codeCells = [];
        notebook = notebook || this.notebookTracker.currentWidget.content;
        notebook.widgets.forEach((cell) => {
            if (cell.model.type === 'code') {
                if (!selectedOnly || notebook.isSelectedOrActive(cell)) {
                    codeCells.push(cell);
                }
            }
        });
        return codeCells;
    }
    getNotebookType() {
        if (!this.notebookTracker.currentWidget) {
            return null;
        }
        const metadata = this.notebookTracker.currentWidget.content.model.sharedModel.metadata;
        ;
        if (!metadata) {
            return null;
        }
        // prefer kernelspec language
        // @ts-ignore
        if (metadata.kernelspec && metadata.kernelspec.language) {
            // @ts-ignore
            return metadata.kernelspec.language.toLowerCase();
        }
        // otherwise, check language info code mirror mode
        // @ts-ignore
        if (metadata.language_info && metadata.language_info.codemirror_mode) {
            // @ts-ignore
            return metadata.language_info.codemirror_mode.name.toLowerCase();
        }
        return null;
    }
    getDefaultFormatters(config) {
        const notebookType = this.getNotebookType();
        if (notebookType) {
            const defaultFormatter = config.preferences.default_formatter[notebookType];
            if (defaultFormatter instanceof Array) {
                return defaultFormatter;
            }
            else if (defaultFormatter !== undefined) {
                return [defaultFormatter];
            }
        }
        return [];
    }
    async getFormattersToUse(config, formatter) {
        const defaultFormatters = this.getDefaultFormatters(config);
        const formattersToUse = formatter !== undefined ? [formatter] : defaultFormatters;
        if (formattersToUse.length === 0) {
            await showErrorMessage('Jupyterlab Code Formatter Error', 'Unable to find default formatters to use, please file an issue on GitHub.');
        }
        return formattersToUse;
    }
    async applyFormatters(selectedCells, formattersToUse, config) {
        var _a;
        for (const formatterToUse of formattersToUse) {
            if (formatterToUse === 'noop' || formatterToUse === 'skip') {
                continue;
            }
            const currentTexts = selectedCells.map(cell => cell.model.sharedModel.source);
            const formattedTexts = await this.formatCode(currentTexts, formatterToUse, config[formatterToUse], true, config.cacheFormatters);
            const showErrors = !((_a = config.suppressFormatterErrors) !== null && _a !== void 0 ? _a : false);
            for (let i = 0; i < selectedCells.length; ++i) {
                const cell = selectedCells[i];
                const currentText = currentTexts[i];
                const formattedText = formattedTexts.code[i];
                const cellValueHasNotChanged = cell.model.sharedModel.source === currentText;
                if (cellValueHasNotChanged) {
                    if (formattedText.error) {
                        if (showErrors) {
                            await showErrorMessage('Jupyterlab Code Formatter Error', formattedText.error);
                        }
                    }
                    else {
                        cell.model.sharedModel.source = formattedText.code;
                    }
                }
                else {
                    if (showErrors) {
                        await showErrorMessage('Jupyterlab Code Formatter Error', `Cell value changed since format request was sent, formatting for cell ${i} skipped.`);
                    }
                }
            }
        }
    }
    async formatCells(selectedOnly, config, formatter, notebook) {
        if (this.working) {
            return;
        }
        try {
            this.working = true;
            const selectedCells = this.getCodeCells(selectedOnly, notebook);
            if (selectedCells.length === 0) {
                this.working = false;
                return;
            }
            const formattersToUse = await this.getFormattersToUse(config, formatter);
            await this.applyFormatters(selectedCells, formattersToUse, config);
        }
        catch (error) {
            await showErrorMessage('Jupyterlab Code Formatter Error', error);
        }
        this.working = false;
    }
    applicable(formatter, currentWidget) {
        const currentNotebookWidget = this.notebookTracker.currentWidget;
        // TODO: Handle showing just the correct formatter for the language later
        return currentNotebookWidget && currentWidget === currentNotebookWidget;
    }
}
export class JupyterlabFileEditorCodeFormatter extends JupyterlabCodeFormatter {
    constructor(client, editorTracker) {
        super(client);
        this.editorTracker = editorTracker;
    }
    formatAction(config, formatter) {
        if (this.working) {
            return;
        }
        const editorWidget = this.editorTracker.currentWidget;
        this.working = true;
        const editor = editorWidget.content.editor;
        const code = editor.model.sharedModel.source;
        this.formatCode([code], formatter, config[formatter], false, config.cacheFormatters)
            .then(data => {
            if (data.code[0].error) {
                void showErrorMessage('Jupyterlab Code Formatter Error', data.code[0].error);
                this.working = false;
                return;
            }
            this.editorTracker.currentWidget.content.editor.model.sharedModel.source =
                data.code[0].code;
            this.working = false;
        })
            .catch(error => {
            this.working = false;
            void showErrorMessage('Jupyterlab Code Formatter Error', error);
        });
    }
    applicable(formatter, currentWidget) {
        const currentEditorWidget = this.editorTracker.currentWidget;
        // TODO: Handle showing just the correct formatter for the language later
        return currentEditorWidget && currentWidget === currentEditorWidget;
    }
}
//# sourceMappingURL=formatter.js.map