import { INotebookTracker } from '@jupyterlab/notebook';
import { ICommandPalette, ToolbarButton } from '@jupyterlab/apputils';
import { ISettingRegistry } from '@jupyterlab/settingregistry';
import { IMainMenu } from '@jupyterlab/mainmenu';
import { IEditorTracker } from '@jupyterlab/fileeditor';
import JupyterlabCodeFormatterClient from './client';
import { JupyterlabFileEditorCodeFormatter, JupyterlabNotebookCodeFormatter } from './formatter';
import { DisposableDelegate } from '@lumino/disposable';
import { Constants } from './constants';
import { LabIcon } from '@jupyterlab/ui-components';
class JupyterLabCodeFormatter {
    constructor(app, tracker, palette, settingRegistry, menu, editorTracker) {
        this.app = app;
        this.tracker = tracker;
        this.editorTracker = editorTracker;
        this.palette = palette;
        this.settingRegistry = settingRegistry;
        this.menu = menu;
        this.client = new JupyterlabCodeFormatterClient();
        this.notebookCodeFormatter = new JupyterlabNotebookCodeFormatter(this.client, this.tracker);
        this.fileEditorCodeFormatter = new JupyterlabFileEditorCodeFormatter(this.client, this.editorTracker);
        this.setupSettings().then(() => {
            this.setupAllCommands();
            this.setupContextMenu();
            this.setupWidgetExtension();
        });
        console.log('222wat');
    }
    createNew(nb, context) {
        const button = new ToolbarButton({
            tooltip: 'Format notebook',
            icon: new LabIcon({
                name: Constants.FORMAT_ALL_COMMAND,
                svgstr: Constants.ICON_FORMAT_ALL_SVG
            }),
            onClick: async () => {
                await this.notebookCodeFormatter.formatAllCodeCells(this.config, undefined, nb.content);
            }
        });
        nb.toolbar.insertAfter('cellType', this.app.commands.label(Constants.FORMAT_ALL_COMMAND), button);
        context.saveState.connect(this.onSave, this);
        return new DisposableDelegate(() => {
            button.dispose();
        });
    }
    async onSave(context, state) {
        if (state === 'started' && this.config.formatOnSave) {
            await this.notebookCodeFormatter.formatAllCodeCells(this.config);
        }
    }
    setupWidgetExtension() {
        this.app.docRegistry.addWidgetExtension('Notebook', this);
    }
    setupContextMenu() {
        this.app.contextMenu.addItem({
            command: Constants.FORMAT_COMMAND,
            selector: '.jp-Notebook'
        });
    }
    setupAllCommands() {
        this.client
            .getAvailableFormatters(this.config.cacheFormatters)
            .then(data => {
            const formatters = JSON.parse(data).formatters;
            const menuGroup = [];
            Object.keys(formatters).forEach(formatter => {
                if (formatters[formatter].enabled) {
                    const command = `${Constants.PLUGIN_NAME}:${formatter}`;
                    this.setupCommand(formatter, formatters[formatter].label, command);
                    menuGroup.push({ command });
                }
            });
            this.menu.editMenu.addGroup(menuGroup);
        });
        this.app.commands.addCommand(Constants.FORMAT_COMMAND, {
            execute: async () => {
                await this.notebookCodeFormatter.formatSelectedCodeCells(this.config);
            },
            // TODO: Add back isVisible
            label: 'Format cell'
        });
        this.app.commands.addCommand(Constants.FORMAT_ALL_COMMAND, {
            execute: async () => {
                await this.notebookCodeFormatter.formatAllCodeCells(this.config);
            },
            iconClass: Constants.ICON_FORMAT_ALL,
            iconLabel: 'Format notebook'
            // TODO: Add back isVisible
        });
    }
    async setupSettings() {
        const settings = await this.settingRegistry.load(Constants.SETTINGS_SECTION);
        const onSettingsUpdated = (jsettings) => {
            this.config = jsettings.composite;
        };
        settings.changed.connect(onSettingsUpdated);
        onSettingsUpdated(settings);
    }
    setupCommand(name, label, command) {
        this.app.commands.addCommand(command, {
            execute: async () => {
                for (const formatter of [
                    this.notebookCodeFormatter,
                    this.fileEditorCodeFormatter
                ]) {
                    if (formatter.applicable(name, this.app.shell.currentWidget)) {
                        await formatter.formatAction(this.config, name);
                    }
                }
            },
            isVisible: () => {
                for (const formatter of [
                    this.notebookCodeFormatter,
                    this.fileEditorCodeFormatter
                ]) {
                    if (formatter.applicable(name, this.app.shell.currentWidget)) {
                        return true;
                    }
                }
                return false;
            },
            label
        });
        this.palette.addItem({ command, category: Constants.COMMAND_SECTION_NAME });
    }
}
/**
 * Initialization data for the jupyterlab_code_formatter extension.
 */
const plugin = {
    id: Constants.PLUGIN_NAME,
    autoStart: true,
    requires: [
        ICommandPalette,
        INotebookTracker,
        ISettingRegistry,
        IMainMenu,
        IEditorTracker
    ],
    activate: (app, palette, tracker, settingRegistry, menu, editorTracker) => {
        new JupyterLabCodeFormatter(app, tracker, palette, settingRegistry, menu, editorTracker);
        console.log('JupyterLab extension jupyterlab_code_formatter is activated!');
    }
};
export default plugin;
//# sourceMappingURL=index.js.map